@@ -2,7 +2,7 @@
  * L.Control.Scroll handles scrollbars
  */
 
-/* global $ */
+/* global $ clearTimeout setTimeout */
 L.Control.Scroll = L.Control.extend({
 
 	onAdd: function (map) {
@@ -25,6 +25,7 @@ L.Control.Scroll = L.Control.extend({
 		map.on('updaterowcolumnheaders', this._onUpdateRowColumnHeaders, this);
 
 		var control = this;
+		var autoHideTimeout = null;
 		$('.scroll-container').mCustomScrollbar({
 			axis: 'yx',
 			theme: 'minimal-dark',
@@ -33,9 +34,20 @@ L.Control.Scroll = L.Control.extend({
 			callbacks:{
 				onScroll: function() {
 					control._onScrollEnd(this);
+					if (autoHideTimeout)
+						clearTimeout(autoHideTimeout);
+					autoHideTimeout = setTimeout(function() {
+						//	$('.mCS-autoHide > .mCustomScrollBox ~ .mCSB_scrollTools').css({opacity: 0, 'filter': 'alpha(opacity=0)', '-ms-filter': 'alpha(opacity=0)'});
+						$('.mCS-autoHide > .mCustomScrollBox ~ .mCSB_scrollTools').removeClass('loleaflet-scrollbar-show');
+					}, 2000);
 				},
 				whileScrolling: function() {
 					control._onScroll(this);
+
+					// autoHide feature doesn't work because plugin relies on hovering on scroll container
+					// and we have a mock scroll container whereas the actual user hovering happens only on
+					// real document. Change the CSS rules manually to simulate autoHide feature.
+					$('.mCS-autoHide > .mCustomScrollBox ~ .mCSB_scrollTools').addClass('loleaflet-scrollbar-show');
 				},
 				onUpdate: function() {
 					console.debug('mCustomScrollbar: onUpdate:');
