@@ -18,7 +18,8 @@ L.Map = L.Evented.extend({
 		defaultZoom: 10,
 		tileWidthTwips: 3840,
 		tileHeightTwips: 3840,
-		urlPrefix: 'lool'
+		urlPrefix: 'lool',
+		wopiSrc: ''
 	},
 
 	lastActiveTime: Date.now(),
@@ -72,6 +73,7 @@ L.Map = L.Evented.extend({
 		this._enabled = true;
 		this._debugAlwaysActive = false; // disables the dimming / document inactivity when true
 		this._serverRecycling = false;
+		this._documentIdle = false;
 
 		vex.dialogID = -1;
 
@@ -116,6 +118,9 @@ L.Map = L.Evented.extend({
 			if (!this.initComplete) {
 				this._fireInitComplete('doclayerinit');
 			}
+			if (this._docLayer._docType == 'text') {
+				L.control.ruler({position:'topleft'}).addTo(this);
+			}
 		});
 		this.on('updatetoolbarcommandvalues', function(e) {
 			if (this.initComplete) {
@@ -134,6 +139,7 @@ L.Map = L.Evented.extend({
 
 		// View info (user names and view ids)
 		this._viewInfo = {};
+		this._viewInfoByUserName = {};
 
 		// View color map
 		this._viewColors = {};
@@ -141,10 +147,26 @@ L.Map = L.Evented.extend({
 		// This becomes true if document was ever modified by the user
 		this._everModified = false;
 
+		// Document is completely loaded or not
+		this._docLoaded = false;
+
 		this.on('commandstatechanged', function(e) {
 			if (e.commandName === '.uno:ModifiedStatus')
 				this._everModified = this._everModified || (e.state === 'true');
 		}, this);
+
+		this.on('docloaded', function(e) {
+			this._docLoaded = e.status;
+			if (this._docLoaded) {
+				// so that dim timer starts from now()
+				this.lastActiveTime = Date.now();
+				if (!document.hasFocus()) {
+					this._deactivate();
+				} else {
+					this._activate();
+				}
+			}
+		}, this);
 	},
 
 	loadDocument: function() {
@@ -164,14 +186,20 @@ L.Map = L.Evented.extend({
 
 	addView: function(viewInfo) {
 		this._viewInfo[viewInfo.id] = viewInfo;
-		this.fire('postMessage', {msgId: 'View_Added', args: {ViewId: viewInfo.id, UserId: viewInfo.userid, UserName: viewInfo.username, Color: L.LOUtil.rgbToHex(viewInfo.color), ReadOnly: viewInfo.readonly}});
+		if (viewInfo.userextrainfo !== undefined && viewInfo.userextrainfo.avatar !== undefined) {
+			this._viewInfoByUserName[viewInfo.username] = viewInfo;
+		}
+		this.fire('postMessage', {msgId: 'View_Added', args: {ViewId: viewInfo.id, UserId: viewInfo.userid, UserName: viewInfo.username, UserExtraInfo: viewInfo.userextrainfo, Color: L.LOUtil.rgbToHex(viewInfo.color), ReadOnly: viewInfo.readonly}});
 
 		// Fire last, otherwise not all events are handled correctly.
-		this.fire('addview', {viewId: viewInfo.id, username: viewInfo.username, readonly: this.isViewReadOnly(viewInfo.id)});
+		this.fire('addview', {viewId: viewInfo.id, username: viewInfo.username, extraInfo: viewInfo.userextrainfo, readonly: this.isViewReadOnly(viewInfo.id)});
+
+		this.updateAvatars();
 	},
 
 	removeView: function(viewid) {
 		var username = this._viewInfo[viewid].username;
+		delete this._viewInfoByUserName[this._viewInfo[viewid].username];
 		delete this._viewInfo[viewid];
 		this.fire('postMessage', {msgId: 'View_Removed', args: {ViewId: viewid}});
 
@@ -187,6 +215,17 @@ L.Map = L.Evented.extend({
 		return this;
 	},
 
+	updateAvatars: function() {
+		if (this._docLayer && this._docLayer._annotations && this._docLayer._annotations._items) {
+			for (var idxAnno in this._docLayer._annotations._items) {
+				var username = this._docLayer._annotations._items[idxAnno]._data.author;
+				if (this._viewInfoByUserName[username]) {
+					$(this._docLayer._annotations._items[idxAnno]._authorAvatarImg).attr('src', this._viewInfoByUserName[username].userextrainfo.avatar);
+				}
+			}
+		}
+	},
+
 	showBusy: function(label, bar) {
 		// If document is already loaded, ask the toolbar widget to show busy
 		// status on the bottom statusbar
@@ -782,7 +821,7 @@ L.Map = L.Evented.extend({
 	},
 
 	_activate: function () {
-		if (this._serverRecycling) {
+		if (this._serverRecycling || this._documentIdle) {
 			return;
 		}
 
@@ -795,7 +834,7 @@ L.Map = L.Evented.extend({
 				console.debug('sending useractive');
 				this._socket.sendMessage('useractive');
 				this._active = true;
-				if (this._docLayer) {
+				if (this._doclayer) {
 					this._docLayer._onMessage('invalidatetiles: EMPTY', null);
 				}
 				if (vex.dialogID > 0) {
@@ -806,7 +845,7 @@ L.Map = L.Evented.extend({
 					return vex.close(id);
 				}
 			} else {
-				this._socket.initialize(this);
+				this.loadDocument();
 			}
 		}
 
@@ -828,12 +867,18 @@ L.Map = L.Evented.extend({
 		this._active = false;
 		clearTimeout(vex.timer);
 
+		var message = '';
+		var map = this;
+		if (!map['wopi'].DisableInactiveMessages) {
+			message = _('Inactive document - please click to resume editing');
+		}
+
 		var options = $.extend({}, vex.defaultOptions, {
 			contentCSS: {'background':'rgba(0, 0, 0, 0)',
-				     'font-size': 'xx-large',
+			             'font-size': 'xx-large',
 				     'color': '#fff',
 				     'text-align': 'center'},
-			content: _('Inactive document - please click to resume editing')
+			content: message
 		});
 		options.id = vex.globalID;
 		vex.dialogID = options.id;
@@ -845,7 +890,6 @@ L.Map = L.Evented.extend({
 			vex: options
 		});
 
-		var map = this;
 		options.$vex.bind('click.vex', function(e) {
 			console.debug('_dim: click.vex function');
 			return map._activate();
@@ -868,7 +912,8 @@ L.Map = L.Evented.extend({
 
 	_dimIfInactive: function () {
 		console.debug('_dimIfInactive: diff=' + (Date.now() - this.lastActiveTime));
-		if ((Date.now() - this.lastActiveTime) >= this.options.idleTimeoutSecs * 1000) {
+		if (this._docLoaded && // don't dim if document hasn't been loaded yet
+		    (Date.now() - this.lastActiveTime) >= this.options.idleTimeoutSecs * 1000) {
 			this._dim();
 		} else {
 			this._startInactiveTimer();
@@ -876,7 +921,7 @@ L.Map = L.Evented.extend({
 	},
 
 	_startInactiveTimer: function () {
-		if (this._serverRecycling) {
+		if (this._serverRecycling || this._documentIdle || !this._docLoaded) {
 			return;
 		}
 
@@ -889,7 +934,7 @@ L.Map = L.Evented.extend({
 	},
 
 	_deactivate: function () {
-		if (this._serverRecycling) {
+		if (this._serverRecycling || this._documentIdle || !this._docLoaded) {
 			return;
 		}
 
@@ -1011,6 +1056,12 @@ L.Map = L.Evented.extend({
 			}
 		}
 
+		// we need to keep track about the last action, this
+		// will help us to avoid wrongly removing the editor
+		if (type === 'keypress') {
+			this.lastActionByUser = true;
+		}
+
 		// we need to keep track if we have entered/left the map
 		this._mouseEnteringLeaving = false;
 		// mouse leaving the map ?
